function eq = gausslab(x,iparam,target,ef); clc
% Calculates the Excess Supply in a life-cycle model with leisure using a
% Gauss-Seidel Algorithm to compute the optimal decision rules.
warning off MATLAB:divideByZero

% Define unkowns of the equilibrium
rss   = x(1);
debt  = x(2);
beta  = x(3);
taul  = x(4);
gamma = x(5);
delta = x(6);
trss  = x(7);

% Define parameter values
I     = iparam(1);
ir    = iparam(2);
tauk  = iparam(3);
Gov   = iparam(4);
theta = iparam(5);
alpha = iparam(6);
tauss = iparam(7);
nss   = iparam(8);
tauc  = iparam(9);

% Initial guess consumers problem
ass      = zeros(I+1,1);        % Savings sequence with terminal condition
ass(1)   = 0.0;                 % Initial asset distribution
abar     = -inf;                % Lower bound on assets
cons     = zeros(I,1);          % Consumption over life-cycle
lab      = zeros(I,1);          % Labor supply over life-cycle

% Updated guess solving the consumer problem
consn    = zeros(I,1);          % Updated consumption over life-cycle
labn     = zeros(I,1);          % Updated labor supply over life-cycle
assn     = zeros(I+1,1);        % Updated savings sequence
assn(1)  = ass(1);              % Initial condition

% Initial tolerance criterium for different markets
dif      = 10;
count    = 0;

% Initial guess on prices (interest rate and wage rate)
wss  = (1.0 - alpha)*theta*(alpha*theta/(rss+delta))^(alpha/(1.0-alpha));
rate = (1.0 + rss*(1.0-tauk)); 
wage = (1.0 - taul)*(1.0 - tauss)*wss;
    
while dif>1e-4

    % Computes asset decisions for workers
    for k=1:ir-2
        assets(k+1)= (beta/(1.0+beta))*(wage*ef(k) + rate*ass(k)) + ((ass(k+2) - wage*ef(k+1))/(rate*(1.0+beta)));
        assn(k+1)  = max(abar,assets(k+1));
    end 
    
    % Computes asset decisions first-period retirement
        assets(ir) = (beta*gamma/(1.0+beta*gamma))*(wage*ef(ir-1) + rate*ass(ir-1)) + ((ass(ir+1) - trss)/(rate*(1.0+gamma*beta)));
        assn(ir)   = max(abar,assets(k+1));               
        
    % Computes asset decisions for retirees
    for k=ir:I-1
        assets(k+1)= (beta/(1.0+beta))*(rate*ass(k)+trss) + ((ass(k+2)-trss)/(rate*(1.0+beta)));
        assn(k+1)  = max(abar,assets(k+1));        
    end       
    
    % Computes consumption and labor supply decisions for workers
    for k=1:ir-1;
        consn(k) = gamma*(wage*ef(k) + rate*assn(k) - assn(k+1))/(1+tauc);
        labor(k) = 1-(1.0-gamma)*consn(k)/(gamma*wage*ef(k)/(1+tauc));
        labn(k)  = max(0,labor(k));
    end
    
     % Computes consumption and labor supply decisions for retirees
     for k=ir:I;
        consn(k) = (rate*assn(k) + trss - assn(k+1))/(1+tauc);
        labn(k)  = 0.0;
    end    
    
    if labor<0; disp('   Labor Supply is NEGATIVE for some cohort'); end
    
    dif   = max(abs(assn-ass)./ass);    
    ass   = assn;
    cons  = consn;
    lab   = labn;
    count = count + 1; 
%    disp(['   Iterations: ',num2str(count),' Error Euler Eq.: ',num2str(dif)])
end

% Calculate aggregate capital and labor supply
  mu   = ((1+nss).^-(0:1:I-1)./(sum((1+nss).^-(0:1:I-1))))';
  Kss  = ass(1:I)'*mu - debt;
  Nss  = ef'*(mu.*lab);
  
% Calculate macro aggregates  
  Yss  = theta*Kss^(alpha)*Nss^(1.0-alpha);
  Css  = (cons'*mu)/Yss;
  Xss  = (nss+delta)*(Kss/Yss);
  RC   = (1.0 - (Css + Xss + Gov));
      
% Calculate Government Budget Constraint  
  Rev   = (tauc*cons'*mu + tauk*rss*Kss + taul*wss*Nss);
  debtn = (1.0/(rate-1.0))*(Rev - Gov*Yss);
  trssnew  = tauss*wss*Nss*sum(mu(1:ir-1))/sum(mu(ir:I));
  
% Calculate new equilibrium prices   
  rnew  = alpha*theta*(Nss/Kss)^(1.0-alpha)-delta;

% Compute equilibrium in the capital markets and government budget constraint
  eq     = zeros(6,1);   
  eq(1)  = ((rnew  - rss)/rss);
  eq(2)  = RC;
  eq(3)  = Kss/Yss - target(1);
  eq(4)  = debt/Yss - target(2);
  eq(5)  = mean(lab) - target(3);
  eq(6)  = Xss - target(4);  
  eq(7)  = trssnew - trss;

  if abs(sum(eq))<1e-4
    disp(' ');disp(' ')
    disp([' EQUILIBRIUM RESIDUALS:           ']);
    disp([' Market Clearing Capital Market:  ',num2str(eq(1))])
    disp([' Market Clearing Goods Market:    ',num2str(eq(2))])
    disp([' market Clearing SS Budget:       ',num2str(eq(7))]);disp(' ')
    disp([' ESTIMATION RESIDUALS:            ']);
    disp([' Capital-Output Ratio:            ',num2str(eq(3))])
    disp([' Debt-Output Ratio:               ',num2str(eq(4))])
    disp([' Investment  to Output Ratio:     ',num2str(eq(6))])
    disp([' Average Hours Worked:            ',num2str(eq(5))]);disp(' ');
    disp([' MOMENT TARGETS:                  ']);
    disp([' Consumption to Output Ratio:     ',num2str(Css)])
    disp([' Investment  to Output Ratio:     ',num2str(Xss)])
    disp([' Gov. Cons.  to Output Ratio:     ',num2str(Gov)])
    disp([' Capital-Output Ratio:            ',num2str(Kss/Yss)])
    disp([' Debt-Output Ratio:               ',num2str(debt/Yss)])
    disp([' Average Hours Worked:            ',num2str(mean(labn))]);disp(' ');
    disp([' ESTIMATION PARAMETERS:           '])
    disp([' Equilibrium interest Rate:       ',num2str(rss)]);    
    disp([' Discount Rate:                   ',num2str(beta)])
    disp([' Consumption Share:               ',num2str(gamma)])
    disp([' Depreciation Rate:               ',num2str(delta)])
    disp([' Capital-Income Tax (exogenous):  ',num2str(tauk)])
    disp([' Labor-income Tax:                ',num2str(taul)])
    disp([' Social Security transfer:        ',num2str(trss)])
    
  end 

% Save initial steady state decision rules
save ststSS ass cons lab
save aggregates rss wss wage tauk taul tauss tauc trss debt Yss
