% Computes the transition path in an OG model with endogenous labor supply
clc; clear all; diary('Results.txt'); close all

% MODEL PARAMETERS
% Households
I      = 65;                  % Life time
ir     = 45;                  % Retirement age 
T      = 35;                  % Periods transition path
nss    = 0.012;               % population growth rate
% Technology
tauc   = 0.05;                % Consumption tax
tauk   = 0.33;                 % Capital-income taxes
tauss  = 0.105;                % Social Security contributions
Gov    = 0.2;                 % Government expenditure
theta  = 1.0;                 % Total factor productivity
alpha  = 0.3;                 % Capital share
ef     = zeros(I,1);

ef(1:ir,1) = [
    0.3366;    0.3678;    0.4007;    0.4353;    0.4714;    0.5091;    0.5482;
    0.5886;    0.6301;    0.6726;    0.7159;    0.7597;    0.8038;    0.8481;
    0.8922;    0.9358;    0.9787;    1.0206;    1.0611;    1.1001;    1.1372;
    1.1721;    1.2046;    1.2343;    1.2611;    1.2848;    1.3050;    1.3218;
    1.3348;    1.3440;    1.3494;    1.3508;    1.3483;    1.3419;    1.3316;
    1.3176;    1.2999;    1.2787;    1.2542;    1.2266;    1.1961;    1.1629;
    1.1274;    1.0898;    1.0503];

% MODEL TARGETS
target(1)  =  3.00;           % Capital-output ratio (Kss/Yss)
target(2)  =  0.50;           % Debt-output ratio (debt/Yss)
target(3)  =  0.33;           % Average hours worked
target(4)  =  0.16;           % Investment-output ratio (Xss)

% INITIAL STEADY STATE
% Solves the initial steady state and estimates equilibrium targets
% iguess = [ rss   debt  beta taul   gamma  delta SSsize ]
iguess   = [.0587    .3    .983  .23   0.46  .04133    0.1371]';
iparam   = [I ir tauk Gov theta alpha tauss nss tauc]';

xi = fsolve(@olgSS,iguess,optimset('fsolve'),iparam,target,ef);

    irss  = xi(1);            % Initial interest rate  
    idebt = xi(2);            % Initial level of government debt
    beta  = xi(3);            % Household discount rate 
    tauli = xi(4);            % Capital income tax
    gamma = xi(5);            % Consumption share utility
    delta = xi(6);            % Depreciation rate
    trss  = xi(7);            % Social Security pensions    

diary off

% COMPUTE GENERATIONAL ACCOUNTS
% Load optimal decision rules and aggregates along the transition path 
load ststSS; 
load aggregates
trsst = [zeros(ir-1,1); ones(I+1-ir,1)*trss];

% Net present value of pensions for all ages
for age=1:I
npvpens(age)=0.0;
ltinc(age)  = wss*ef(age)*lab(age)+ trsst(age);   
    for ii=age:I
    npvpens(age)= npvpens(age)+trsst(ii)*( 1/(1.0+rss*(1.0-tauk))^(ii-age));
    ltinc(age)  = ltinc(age)  + (wss*ef(ii)*lab(ii)+ trsst(ii))*...
                  ( 1/(1.0+rss*(1.0-tauk))^(ii-age)); 
    end
end

% Compute asset distribution final steady state
assn    = zeros(I,1);
assn(I) = (1+tauc)*cons(I)/(1.0+rss*(1.0-tauk));

for age=I-1:-1:2
    assn(age) = ((1+tauc)*cons(age)+assn(age+1)-wage*ef(age)*lab(age))/(1.0+rss*(1.0-tauk));
end

assn(1)   = npvpens(1);
assn(I+1) = 0.0;

%-------------------------------------------------------------------------
% Compute GA Baseline with SS
%-------------------------------------------------------------------------
rss0 = 0.05998;
rss1 = rss0+0.01;
rss2 = rss0-0.01;

tb   = zeros(I,1);
tbn  = zeros(I,1);
yi   = zeros(I,1);
yin  = zeros(I,1);

tb(1)  = tauc*cons(1) + (taul+tauss)*wss*ef(1)*lab(1);
tbn(1) = tauc*cons(1) + (taul+tauss)*wss*ef(1)*lab(1)-npvpens(1);
yi(1)  = wss*ef(1)*lab(1)+ trsst(1);
yin(1) = wss*ef(1)*lab(1)+ npvpens(1);

for age=2:I
  tb(age)  = tauc*cons(age) + (taul+tauss)*wss*ef(age)*lab(age) + tauk*rss*ass(age)- trsst(age);
  tbn(age) = tauc*cons(age) + (taul+tauss)*wss*ef(age)*lab(age) + tauk*rss*assn(age);
  yi(age)  = wss*ef(age)*lab(age)+ trsst(age);
  yin(age) = wss*ef(age)*lab(age);  
end


debtss   = sum(assn)-sum(ass);
totdebt  = debt+debtss;
debt2gdp = totdebt/Yss;

for age=1:I
    
    ga(age)   = 0.0;
    ga1(age)  = 0.0; 
    ga2(age)  = 0.0; 
   
    for ii=age:I
        ga(age)  = ga(age)+tb(ii)*(1/(1.0+rss0))^(ii-age);
        ga1(age) = ga1(age)+tb(ii)*(1/(1.0+(rss1)))^(ii-age);
        ga2(age) = ga2(age)+tb(ii)*(1/(1.0+(rss2)))^(ii-age);        
    end
end

%-------------------------------------------------------------------------
% Compute GA New Policy with no SS
%-------------------------------------------------------------------------
for age=1:I
    gant(age) = tauc*cons(age) + (taul+tauss)*wss*ef(age)*lab(age) + tauk*rss*assn(age)-npvpens(age);
    ga1nt(age)= tauc*cons(age) + (taul+tauss)*wss*ef(age)*lab(age) + tauk*rss*assn(age)-npvpens(age);
    ga2nt(age)= tauc*cons(age) + (taul+tauss)*wss*ef(age)*lab(age) + tauk*rss*assn(age)-npvpens(age);
    
  for ii=age+1:I
     gant(age)  = gant(age)+tbn(ii)*(1/(1.0+rss0))^(ii-age);   
     ga1nt(age) = ga1nt(age)+tbn(ii)*(1/(1.0+(rss1)))^(ii-age);
     ga2nt(age) = ga2nt(age)+tbn(ii)*(1/(1.0+(rss2)))^(ii-age);
     
  end
end

t   = 1930:1:1994;
age = 20:1:66+19;

figure; plot(age,ass./Yss,'LineWidth',3); hold;
plot(age,assn./Yss,'r--','LineWidth',3)
plot(age,zeros(66,1),'k','LineWidth',1)
set(gca,'LineWidth',1,'FontWeight','Bold','FontSize',12,'Box','off');
legend('Unfunded: Policy A','Funded: Policy B',2);legend('Boxoff')
ylabel('Assets (relative to yearly income)')
axis([20 85 -4 16])
print('-dpdf','-r200','GA_Assets')
print('-dtiff','-r200','GA_Assets')

figure; plot(age(1:65),tb./Yss,'LineWidth',3); hold;
plot(age(1:65),tbn./Yss,'r--','LineWidth',3)
plot(age(1:65),zeros(65,1),'k','LineWidth',1)
set(gca,'LineWidth',1,'FontWeight','Bold','FontSize',12,'Box','off');
legend('Unfunded: Policy A','Funded: Policy B',0); legend('Boxoff')
ylabel('Net Taxes (relative to yearly income)')
axis([20 85 -.8 .8])
print('-dpdf','-r200','GA_Taxes')
print('-dtiff','-r200','GA_Taxes')

err0 = (rot90(ga)-rot90(gant));
err1 = (rot90(ga1)-rot90(ga1nt));
err2 = (rot90(ga2)-rot90(ga2nt));

[val0,pos0] = max(abs(err0)*100);
[val1,pos1] = max(abs(err1)*100);
[val2,pos2] = max(abs(err2)*100);

figure;
plot(t,rot90(ga1)./Yss,'LineWidth',3); hold
plot(t,rot90(ga1nt)./Yss,'r--','LineWidth',3)
plot(t,zeros(65,1),'k','LineWidth',1)
set(gca,'LineWidth',1,'FontWeight','Bold','FontSize',12,'Box','off');
title('Intergenerational Discount R=7%')
ylabel('Ga (relative to yearly income)')
legend(['Unfunded: Policy A'],['Funded: Policy B (Bias= ',num2str(val1,3),'%)'],2)
legend('Boxoff')
axis([1930 2000 -2 10])
print('-dpdf','-r200','GA_Case2')
print('-dtiff','-r200','GA_Case2')

figure;
plot(t,rot90(ga2)./Yss,'LineWidth',3); hold
plot(t,rot90(ga2nt)./Yss,'r--','LineWidth',3)
plot(t,zeros(65,1),'k','LineWidth',1)
set(gca,'LineWidth',1,'FontWeight','Bold','FontSize',12,'Box','off');
title('Intergenerational Discount R=5%')
legend('Unfunded: Policy A','Funded: Policy B',2);legend('Boxoff')
legend(['Unfunded: Policy A'],['Funded: Policy B (Bias= ',num2str(val2,3),'%)'],2)
legend('Boxoff')
ylabel('Ga (relative to yearly income)')
axis([1930 2000 -2 10])
print('-dpdf','-r200','GA_Case3')
print('-dtiff','-r200','GA_Case3')

figure;
plot(t,rot90(ga)./Yss,'LineWidth',3); hold
plot(t,rot90(gant)./Yss,'r--','LineWidth',3)
plot(t,zeros(65,1),'k','LineWidth',1)
set(gca,'LineWidth',1,'FontWeight','Bold','FontSize',12,'Box','off');
title('Intergenerational Discount R=6%')
legend(['Unfunded: Policy A'],['Funded: Policy B'],2)
legend('Boxoff')
ylabel('Ga (relative to yearly income)')
axis([1930 2000 -2 10])
print('-dpdf','-r200','GA_Case1')
print('-dtiff','-r200','GA_Case1')